-- On linux, run psql -d actionHubDb -f Upgrade.sql
DO
$do$
BEGIN
-- 1. create new tables
IF NOT EXISTS (SELECT relname from pg_class where relname='alert_definition') THEN
BEGIN
raise notice 'creating alert_definition table';
CREATE TABLE alert_definition
(
  id bigint NOT NULL,
  date_created timestamp without time zone NOT NULL,
  details text NOT NULL,
  event_id bigint NOT NULL,
  last_mod_date timestamp without time zone NOT NULL,
  name character varying(255) NOT NULL,
  phase character varying(255) NOT NULL,
  severity character varying(255) NOT NULL,
  source_name character varying(128) NOT NULL,
  value_stream character varying(255) NOT NULL,
  rule_id bigint,
  CONSTRAINT fk_k2lu4us85dtgd7dol1yfxvqeu FOREIGN KEY (rule_id)
      REFERENCES public.rule (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (
  OIDS=FALSE
);
--
-- Name: alert_definition_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE alert_definition_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: alert_definition_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE alert_definition_id_seq OWNED BY alert_definition.id;

--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY alert_definition ALTER COLUMN id SET DEFAULT nextval('alert_definition_id_seq'::regclass);

--
-- Name: daemon_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY alert_definition
    ADD CONSTRAINT alert_definition_pkey PRIMARY KEY (id);

END;
END IF;

IF NOT EXISTS (SELECT relname from pg_class where relname='alert_instance') THEN
BEGIN
raise notice 'creating alert_instance table';
CREATE TABLE alert_instance
(
  id bigint NOT NULL,
  acknowledged boolean,
  acknowledged_by character varying(128),
  acknowledged_datetime timestamp without time zone,
  date_created timestamp without time zone NOT NULL,
  details text NOT NULL,
  event_id bigint NOT NULL,
  name character varying(255) NOT NULL,
  phase character varying(255) NOT NULL,
  rule_id bigint,
  severity integer NOT NULL,
  source_id character varying(128) NOT NULL,
  source_name character varying(255) NOT NULL,
  value_stream character varying(255) NOT NULL
)
WITH (
  OIDS=FALSE
);
--
-- Name: alert_instance_id_seq; Type: SEQUENCE; Schema: public; Owner: -
--

CREATE SEQUENCE alert_instance_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


--
-- Name: alert_instance_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: -
--

ALTER SEQUENCE alert_instance_id_seq OWNED BY alert_instance.id;

--
-- Name: id; Type: DEFAULT; Schema: public; Owner: -
--

ALTER TABLE ONLY alert_instance ALTER COLUMN id SET DEFAULT nextval('alert_instance_id_seq'::regclass);

--
-- Name: daemon_pkey; Type: CONSTRAINT; Schema: public; Owner: -
--

ALTER TABLE ONLY alert_instance
    ADD CONSTRAINT alert_instance_pkey PRIMARY KEY (id);

END;
END IF;

-- 3. Drop constraints

BEGIN
    raise notice 'Removing NOT NULL constraint for target_tool_id, target_workflow and target_workflow_id in rule table';
    	ALTER TABLE rule ALTER COLUMN target_tool_id DROP NOT NULL;
	ALTER TABLE rule ALTER COLUMN target_workflow DROP NOT NULL;
	ALTER TABLE rule ALTER COLUMN target_workflow_id DROP NOT NULL;
	ALTER TABLE rule_execution_history ALTER COLUMN target_workflow DROP NOT NULL;
	ALTER TABLE rule_execution_history ALTER COLUMN target_name DROP NOT NULL;	
END;

-- 4. Add new columns

IF NOT EXISTS (SELECT column_name 
               FROM information_schema.columns 
               WHERE table_schema='public' and table_name='rule' and column_name='rule_type') THEN
BEGIN
ALTER TABLE rule ADD column rule_type character varying(255) ;
END;
END IF;

IF NOT EXISTS (SELECT column_name 
               FROM information_schema.columns 
               WHERE table_schema='public' and table_name='alert_instance' and column_name='project_id') THEN
BEGIN
ALTER TABLE alert_instance ADD column project_id character varying(32) NOT NULL;
END;
END IF;


IF NOT EXISTS (SELECT column_name 
               FROM information_schema.columns 
               WHERE table_schema='public' and table_name='rule_execution_history' and column_name='alert_instance_id') THEN
BEGIN
ALTER TABLE rule_execution_history ADD column alert_instance_id bigint;
END;
END IF;

IF NOT EXISTS (SELECT column_name 
               FROM information_schema.columns 
               WHERE table_schema='public' and table_name='rule_execution_history' and column_name='alert_instance_name') THEN
BEGIN
ALTER TABLE rule_execution_history ADD column alert_instance_name text;
END;
END IF;


-- 5. Remove columns
ALTER TABLE alert_definition DROP COLUMN IF EXISTS source_name;
ALTER TABLE alert_definition DROP COLUMN IF EXISTS value_stream;
ALTER TABLE alert_definition DROP COLUMN IF EXISTS date_created;
ALTER TABLE alert_definition DROP COLUMN IF EXISTS last_mod_date;


 -- Update null rule_type values to 'Action'
BEGIN
UPDATE rule SET rule_type = 'Action' WHERE rule_type IS NULL; 
END;

-- Set rule_type column to NOT NULL
BEGIN
ALTER TABLE  rule
    ALTER COLUMN rule_type SET NOT NULL;
END;


raise notice 'updating schema version';
-- 5. Update the schema version
UPDATE schema_versions set compat = '1704', version = '1704' 
where ID = 1;
raise notice 'upgrade schema completed';
END
$do$